import javax.swing.*;
import javax.swing.text.*;

import java.awt.Rectangle;

import java.net.UnknownHostException;
import java.io.IOException;

import java.util.StringTokenizer;

import wtalk.WTalk;

class ChatInputHandler {
	private TabbedChat tc = null;
	private JTextPane tp = null;
	private WTalk wt = null;
	private String CurrentRoom = null;
	private Document doc = null;

	public ChatInputHandler(String room, JTextPane t, TabbedChat c, WTalk w) {
		CurrentRoom = room;
		tp = t;	wt = w; tc = c;
		doc = t.getDocument();
	}

	public void printChat(String s) {
		try {
			doc.insertString(doc.getLength(), s, tp.getStyle("client"));
			tp.scrollRectToVisible(new Rectangle(0, tp.getHeight(), 1, 1));
		} catch (BadLocationException ble) {}
	}

	public void doInput(String s) {
		if(s.length() <= 0) return;

		if(s.charAt(0)=='/') doCommand(s.substring(1));
		else if(CurrentRoom != null && s!=null && s.length() > 0)
			doCommand("msg " + CurrentRoom + " " + s);
		else printChat(ErrorMessages.NO_ACTIVE_ROOM);
	}

	public void doCommand(String s) {
		StringTokenizer st = new StringTokenizer(s);
		String com;

		if(st.hasMoreTokens()){
			com = st.nextToken();


			//
			// UI Commands
			//
			if(com.equals("clear")) {
				try {
					doc.remove(0, doc.getLength());
				} catch ( BadLocationException ble) {}
				return;
			} else if (com.equals("close")) {
				String room;
				if(st.hasMoreTokens()) {
					room = st.nextToken();
				} else {
					room = CurrentRoom;
				}

				if(room!=null) {
					int i = tc.getUI().indexOfTab(room);
					if(i>=0) tc.getUI().removeTabAt(i);
				}
				return;	
			} else if (com.equals("open")) {
				if(st.hasMoreTokens()) {
					String room = st.nextToken();
					if(room!=null) {
						int i = tc.getUI().indexOfTab(room);
						if(i<0) tc.addTab(room, JChatPanel.IM_TAB);
					}
				} 
				return;
			}



			//
			// Connection commands
			//
			if(com.equals("server")) {
				if(st.hasMoreTokens()) {
					int port;
					String server = st.nextToken();
					if(st.hasMoreTokens()){
						port = Integer.parseInt(st.nextToken());
						wt.getWTalkProtocol().setServerPort(port);
					}

					wt.getWTalkProtocol().setServer(server);
				}
				return;
			} else  if(com.equals("connect")) {
					String server = wt.getWTalkProtocol().getServer();
					int port = wt.getWTalkProtocol().getServerPort();
					try {
						printChat("Trying to connect: " + server + ":" + port  + "\n"); 
						if(st.hasMoreTokens())
							wt.getWTalkProtocol().connect(st.nextToken());
						else
							wt.getWTalkProtocol().connect();
					} catch (UnknownHostException e) {
						printChat("Unknown Host: " + server + "\n"); 
					} catch (IOException e) {
						printChat("Error Connecting: " + server + ":" + port + "\n");
					}
				return;
			}



			//
			// Connected Only
			//
			if(wt.getWTalkProtocol() == null || !wt.getWTalkProtocol().isConnected()) {
				printChat(ErrorMessages.NOT_CONNECTED); 
				return;
			}





			if(com.equals("msg")) {
				if(st.hasMoreTokens()) {
					String to = st.nextToken();
					if(st.hasMoreTokens())
						wt.getWTalkProtocol().sendMessage(to, s.substring(to.length()+com.length()+2));
				}
			} else if (com.equals("wall")) {
				if(st.hasMoreTokens()) {
					wt.getWTalkProtocol().sendWall(s.substring(com.length()+1));
				}
			} else if (com.equals("nick")) {
				if(st.hasMoreTokens()) {
					String nick = st.nextToken();
					wt.getWTalkProtocol().setName(nick);
				}
			} else if (com.equals("quit")) {
				if(st.hasMoreTokens()) {
					wt.getWTalkProtocol().disconnect(s.substring(com.length()+1));
				} else {
					wt.getWTalkProtocol().disconnect(null);
				}
				 printChat(ErrorMessages.DISCONNECTED);
			} else if(com.equals("join")) {
				if(st.hasMoreTokens()){
					String room=s.substring(com.length()+1);
					wt.getWTalkProtocol().joinChat(room);
					// If there isn't a tab already ...
					if(tc.getUI().indexOfTab(room) < 0)
						tc.addTab(room, JChatPanel.CHAT_TAB);
				} else if(CurrentRoom != null) {
					wt.getWTalkProtocol().joinChat(CurrentRoom);
				} else {
					printChat(ErrorMessages.NO_ROOM_SPECIFIED);
				}
			} else if(com.equals("leave") || com.equals("part")) {
				if(st.hasMoreTokens()) {
					wt.getWTalkProtocol().partChat(s.substring(com.length()+1));
				} else if(CurrentRoom != null) {
					wt.getWTalkProtocol().partChat(CurrentRoom);
				} else {
					printChat(ErrorMessages.NO_ROOM_SPECIFIED);
				}
			} else if (com.equals("list")) {
				if(st.hasMoreTokens()) {
					wt.getWTalkProtocol().chatList(s.substring(com.length()+1));
				} else {
					wt.getWTalkProtocol().chatList(null);
				}
			} else if (com.equals("topic")) {
				if (st.hasMoreTokens()) {
					String room = CurrentRoom==null?st.nextToken():CurrentRoom;
					if(st.hasMoreTokens())
						wt.getWTalkProtocol().setTopic(room, s.substring(room.length() + com.length() +2));
				} else {
					printChat(ErrorMessages.NO_ROOM_SPECIFIED);
				}
			} else if (com.equals("whois") || com.equals("wi")) {
				String who =  null;
				if (st.hasMoreTokens()) {
					who = s.substring(com.length() + 1);
				} else if (CurrentRoom != null) {
					who = CurrentRoom;
				}

				if (who!=null) {
					wt.getWTalkProtocol().whois(who);
				} else {
					printChat(ErrorMessages.NO_USER_SPECIFIED);
				}
			} else if (com.equals("send") || com.equals("raw")) {
				wt.getWTalkProtocol().sendRaw(s.substring(com.length() + 1));
			} else {
				 printChat(ErrorMessages.UNKNOWN_CMD);
			}
		}
	}

}
