import javax.swing.*;
import javax.swing.text.*;
import javax.swing.tree.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.*;

import java.net.*;
import java.io.IOException;

import wtalk.WTalk;
import wtalk.RMsgHandler;

class ChatReader implements RMsgHandler {
	protected TabbedChat tc;

	public ChatReader(){}

	public void setTabs(TabbedChat t){
		tc = t;
	}

	public void handle(String to, String from, String data) {
		Document doc;
		JTextPane tp;
		int i = tc.getUI().indexOfTab(to);

		if(i>=0)
			tp = ((JChatPanel)(tc.getUI().getComponentAt(i))).getJTextPane();
		else
			tp = tc.addTab(to, JChatPanel.CHAT_TAB).getJTextPane();

		// Highlight
		i=tc.getUI().indexOfTab(to);
		if(i!=tc.getUI().getSelectedIndex()){
			tc.getUI().setForegroundAt(i, new Color(100, 0, 0));
		}


		doc= tp.getDocument();

		try {
			doc.insertString(doc.getLength(), from + ": " + data + "\n", tp.getStyle("chat"));
			tp.scrollRectToVisible(new Rectangle(0, tp.getHeight(), 1, 1));
		} catch (BadLocationException ble) {}
	}
}

class WallReader implements RMsgHandler {
	private Document doc;
	private JTextPane tp;

	public WallReader(){}

	public WallReader(JTextPane t){
		setPane(t);
	}

	public void setPane(JTextPane t){
		tp = t; doc = t.getDocument();
	}

	public void handle(String to, String from, String data) {
		try {
			doc.insertString(doc.getLength(), from + ": " + data + "\n", tp.getStyle("wall"));
			tp.scrollRectToVisible(new Rectangle(0, tp.getHeight(), 1, 1));
		} catch (BadLocationException ble) {}
	}
}

class SystemReader implements RMsgHandler {
	private Document doc;
	private JTextPane tp;

	public SystemReader(){}
	
	public SystemReader(JTextPane t){
		setPane(t);
	}

	public void setPane(JTextPane t){
		tp = t; doc = t.getDocument();
	}

	public void handle(String to, String from, String data) {
		try {
			doc.insertString(doc.getLength(), "--" + data + "\n", tp.getStyle("server"));
			tp.scrollRectToVisible(new Rectangle(0, tp.getHeight(), 1, 1));
		} catch (BadLocationException ble) {}
	}
}

class MsgReader implements RMsgHandler {
	protected TabbedChat tc;

	public MsgReader(){}

	public void setTabs(TabbedChat t){
		tc = t;
	}

	public void handle(String to, String from, String data){
		Document doc;
		JTextPane tp;
		boolean isReceiver = (from != null);	// Receiver or Sender?
		String s = isReceiver? from : to;
		int i = tc.getUI().indexOfTab(s);

		if(i>=0)
			tp = ((JChatPanel)(tc.getUI().getComponentAt(i))).getJTextPane();
		else
			tp = tc.addTab(s, JChatPanel.IM_TAB).getJTextPane();

		// Highlight
		i=tc.getUI().indexOfTab(s);
		if(i!=tc.getUI().getSelectedIndex()){
			tc.getUI().setForegroundAt(i, DmcColors.TAB_HIGHLIGHT);
		}
		

		doc = tp.getDocument();

		try {
			if(isReceiver)
				doc.insertString(doc.getLength(), from + ": ", tp.getStyle("usay"));
			else
				doc.insertString(doc.getLength(), tc.getUserName() + ": ", tp.getStyle("isay"));
			doc.insertString(doc.getLength(), data + "\n", tp.getStyle("chat"));
			tp.scrollRectToVisible(new Rectangle(0, tp.getHeight(), 1, 1));
		} catch (BadLocationException ble) {}
		
	}
}





public class TabbedChat {
	final protected JTabbedPane tabbedPane = new JTabbedPane();
	final protected WTalk wt = new WTalk();
	final protected ChatReader cr = new ChatReader();
	final protected WallReader wr = new WallReader();
	final protected SystemReader sr = new SystemReader();
	final protected MsgReader mr = new MsgReader();


	public TabbedChat() {
		initReaders();
		setConnectionType(wtalk.Protocol.USA);
		registerReaders();
		addListeners();
	}

	public void setConnectionType(int i) {

		wt.setWTalkProtocol(i);
	}

	private void initReaders() {
		JChatPanel cp = addTab("System", JChatPanel.SYSTEM_TAB);
		wr.setPane(cp.getJTextPane());
		sr.setPane(cp.getJTextPane());
		cr.setTabs(this);
		mr.setTabs(this);
	}

	private  void registerReaders() {
		wt.getWTalkProtocol().setReceivedMessageHandler(cr);
		wt.getWTalkProtocol().setChatMessageHandler(cr);
		wt.getWTalkProtocol().setPrivateMessageHandler(mr);
		wt.getWTalkProtocol().setWallMessageHandler(wr);
		wt.getWTalkProtocol().setSystemMessageHandler(sr);

	}

	private void addListeners() {
		tabbedPane.addChangeListener(new ChangeListener(){
			public void stateChanged(ChangeEvent e) {
				tabbedPane.setForegroundAt(tabbedPane.getSelectedIndex(), DmcColors.TAB_NORMAL);
			}
		});
	}


	public JTabbedPane getUI() {
		return tabbedPane;
	}

	public String getUserName() {
		return wt.getWTalkProtocol().getName();
	}

	public JChatPanel addTab(String name, int type) {
		JChatPanel pane = new JChatPanel(this, wt, type!=0?name:null);
		
		pane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

		tabbedPane.addTab(name, pane);
		tabbedPane.setForegroundAt(tabbedPane.indexOfTab(name), DmcColors.TAB_NORMAL);

		return pane;
	}

}
