package wtalk.protocols.usa;

import java.io.*;
import java.net.*;

import java.util.StringTokenizer;
import java.util.NoSuchElementException;

import wtalk.RMsgHandler;

class ReceivedListener extends Thread{
	private BufferedReader in=null;
	private RMsgHandler rmh=null;	// generic message handler
	private RMsgHandler rwh = null;	// handles wall messages if !null
	private RMsgHandler rsh = null;	// handles system messages if !null
	private RMsgHandler rph = null;	// handles private messages if !null
	private StatusListener sl = null; //listens for status resposnes

	public ReceivedListener(){
		in = null; rmh = null;
	}

	public void ListenTo(BufferedReader n){
		in = n;
	}

	public void ReportTo(RMsgHandler r){
		rmh = r;
	}

	public void ReportWallTo(RMsgHandler r){
		rwh = r;
	}

	public void ReportSystemTo(RMsgHandler r){
		rsh = r;
	}

	public void ReportPrivateTo(RMsgHandler r){
		rph = r;
	}

	public void setStatusListener(StatusListener s){
		sl = s;
	}

	public void run(){
		String s;
		String target, from, data;

		StringTokenizer st;

		if(in==null) return;

		while(true){

			try{
				s = in.readLine();
			} catch(IOException e){ return; }


			if(s==null) break;

			st = new StringTokenizer(s);

			try{
				target = st.nextToken();
			} catch(NoSuchElementException e){ continue; }

			if(st.hasMoreElements())
				from = st.nextToken();
			else continue;

			if(st.hasMoreElements())
				data = s.substring(target.length() + from.length() + 2);
			else continue;


			// system ping
			if(from.equals("*") && data.equals("*")) continue;

			// status responses
			if(from.equals("+")) {
				sl.gotPositiveResponse(data);
				continue;
			} else if(from.equals("-")) {
			  	continue;
			}


			// system message
			if(from.equals("*")){
				if(rsh!=null)
					rsh.handle(null, null, data);
				else 
					if(rmh!=null) 
						rmh.handle(null, null, data);

				continue;
			}

			// wall message
			if(target.equals("*")){
				if(rwh!=null)
					rwh.handle(null, from, data);
				else 
					if(rmh!=null) 
						rmh.handle(null, from, data);

				continue;
			}


			// normal chat
			if(!(target.charAt(0)=='&' || target.charAt(0)=='#') && rph!=null)
				rph.handle(target,from, data);
			else
				if(rmh!=null) 
					rmh.handle(target, from, data);
			
		}

		in=null;

	}
}

